import CacheItem from "./CacheItem";
import { Buffer } from 'buffer';
var SimpleCache = /** @class */ (function () {
    function SimpleCache(param) {
        this.maxLength = 10;
        this.cacheMap = {};
        this.lastResetTime = new Date();
        this.baseDate = 1000 * 60 * 24;
        var _a = param.maxLength, maxLength = _a === void 0 ? 10 : _a;
        this.maxLength = maxLength;
    }
    SimpleCache.prototype.get = function (key) {
        try {
            if (this.cacheMap[key] !== undefined) {
                return this.cacheMap[key];
                this.resetPriority();
            }
            else {
                throw new Error('no such key');
            }
        }
        catch (e) {
            this.handleError(e);
        }
    };
    SimpleCache.prototype.set = function (key, value) {
        try {
            if (this.cacheMap[key] === undefined) {
                var item = this.formatItem(key, value);
                if (this.isOverflow()) {
                    this.lastResetTime = new Date();
                    this.resetPriority();
                }
                if (!this.isOverflow()) {
                    this.cacheMap[key] = item;
                }
            }
            else {
                this.cacheMap[key].value = Buffer.from(JSON.stringify(value));
                this.resetItem(this.cacheMap[key], new Date(), 1);
            }
        }
        catch (e) {
            this.handleError(e);
        }
    };
    SimpleCache.prototype.empty = function () {
        this.lastResetTime = new Date();
        this.cacheMap = {};
    };
    SimpleCache.prototype.showInfo = function () {
        console.log(JSON.stringify(this.cacheMap));
    };
    SimpleCache.prototype.formatItem = function (key, value) {
        return new CacheItem(key, value);
    };
    SimpleCache.prototype.resetPriority = function () {
        var _this = this;
        Object.keys(this.cacheMap).forEach(function (key) {
            _this.resetItem(_this.cacheMap[key], _this.lastResetTime);
        });
    };
    SimpleCache.prototype.resetItem = function (item, now, up) {
        if (now === void 0) { now = new Date(); }
        if (up === void 0) { up = 0; }
        var createTime = item.createTime, lastModify = item.lastModify, priority = item.priority;
        var x = Math.floor((lastModify.getTime() - createTime.getTime()) / this.baseDate) + 1;
        var y = Math.floor((now.getTime() - createTime.getTime()) / this.baseDate) + 1;
        item.priority = priority * x / y + up;
        item.lastModify = now;
    };
    SimpleCache.prototype.handleError = function (e) {
        console.error(e);
    };
    SimpleCache.prototype.isOverflow = function () {
        return Object.keys(this.cacheMap).length >= this.maxLength;
    };
    return SimpleCache;
}());
export default SimpleCache;
